
import 'package:flutter/material.dart';
import 'package:nirrti/nirrti.dart';
import 'package:nirrti/src/custom_components/custom_radio/view_model.dart';

mixin ViewMixin {
  render(BuildContext context, OwlnextRadioState state) {    
    return LayoutBuilder(
      builder: (BuildContext context, BoxConstraints constraints) {
        return FormField(
          builder: (FormFieldState<bool> formState) {
            return Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Row(
                  children: [
                    Flexible(
                      child: Text(
                        '${state.widget.label} ${state.widget.isMandatory == true ? '*' : ''}',
                        style: Artist().labelTextStyle,
                      ),
                    ),
                    if (state.widget.infoCallback != null) 
                    Padding(
                      padding: const EdgeInsets.only(left: 8.0),
                      child: InkWell(
                        child: MouseRegion(
                          cursor: SystemMouseCursors.click,
                          child: GestureDetector(
                            onTap: () {
                              state.widget.infoCallback!(
                                state.widget.infoCallback,
                                state.widget.label,
                              );
                            },
                            child: Container(
                              width: 17,
                              height: 17,
                              padding: const EdgeInsets.all(2),
                              decoration: BoxDecoration(
                                border: Border.all(
                                  color: Theme.of(context).colorScheme.primary,
                                ),
                                borderRadius: BorderRadius.circular(100)
                              ),
                              child: Icon(
                                Icons.question_mark,
                                color: Theme.of(context).colorScheme.primary,
                                size: 10
                              )
                            )
                          )
                        )
                      ),
                    )
                  ],
                ),
                SizedBox(
                  child: Row(
                    children: [
                      for (CustomRadioOption option in state.widget.options)
                      generateOption(context, state, option),
                    ]
                  )
                ),
                if (formState.errorText != null) 
                const SizedBox(
                  height: 6
                ),
                if (formState.errorText != null) 
                Text(
                  formState.errorText ?? '',
                  style: TextStyle(
                    fontSize: 12,
                    color: Theme.of(context).colorScheme.onError
                  ),
                ),
                if (state.widget.disabledMessage != null) 
                const SizedBox(
                  height: 8
                ),
                if (state.widget.disabledMessage != null)
                Column(
                  children: [
                    OwlnextHelper(
                      message: state.widget.disabledMessage ?? ''
                    ),
                    const SizedBox(
                      height: 16
                    ),
                  ],
                )
              ],
            );
          },
          validator: (value) {
            if (state.widget.isMandatory == true && state.widget.selectedOption == null) {
              return 'Cette information est obligatoire';
            }
        
            return null;
          },
        );
      }
    );
  }

  Widget generateOption(BuildContext context, OwlnextRadioState state, CustomRadioOption option) {
    return InkWell(
      child: MouseRegion(
        cursor: SystemMouseCursors.click,
        child: GestureDetector(
          onTap: () {
            if (state.widget.enabled == true) {
              state.widget.handleChangeSelected(option);
            }
          },
          child: Container(
            margin: EdgeInsets.only(right: Artist().defaultSeparation),
            child: Row(
              children: [
                Container(
                  width: 10,
                  height: 10,
                  decoration: BoxDecoration(
                    color: option.value == state.widget.selectedOption ? Theme.of(context).colorScheme.primary : Colors.transparent,
                    borderRadius: Artist().defaultBorderRadius,
                    border: Border.all(
                      color: Theme.of(context).colorScheme.primary,
                      width: 1,
                    )
                  ),
                ),
                SizedBox(
                  child: Container(
                    padding: Artist().defaultPadding,
                    alignment: Alignment.center,
                    child: Align(
                      child: option.widget,
                    )
                  ),
                ),
              ],
            ),
          ),
        )
      )
    );
  }
}
