import 'dart:developer';

import 'package:uuid/uuid.dart';
import 'generic_list.dart';

class FieldsDetails {
    String? id;
    late String label;
    late String name;
    late FieldType displayType;
    late String valueType;
    late List<FieldsDetails?>? fields;
    late List<GenericList?>? options;
    late bool required;
    late String? value;
    late String? hintText;
    String? suffixText;
    List<TargetResolver>? trigger;

    FieldsDetails({
      this.id,
      required this.label,
      required this.name,
      required this.displayType,
      required this.valueType,
      this.fields,
      this.options,
      required this.required,
      this.value,
      this.hintText,
      this.suffixText,
      this.trigger,
    });

    FieldsDetails.fromJson(dynamic input) {
        try {
          label = input['label'];
          name = input['name'];
          valueType = input['valueType'];
          fields =  input['fields'] != null ? [...input['fields'].map((e) => FieldsDetails.fromJson(e))] : null;
          options =  input['options'] != null ? [...input['options'].map((e) => GenericList.fromJson(e))] : null;
          required = input['required'];
          value = input['value'];
          hintText = input['hintText'];
          suffixText = input['suffixText'];

          for (int i = 0; i < FieldType.values.length; i ++) {
            if (FieldType.values[i].name == input['displayType']) {
              displayType = FieldType.values[i];
            }
          }

          if (input['trigger'] != null) {
            trigger = [...input['trigger'].map((e) => TargetResolver.fromJson(e))];
          }

          id = input['id'] ?? const Uuid().v1();
        } catch(e, stacktrace) {
            log(e.toString());
            log('Function: FieldsDetails.fromJson()');
            log(stacktrace.toString());
        }
    }
}

class TargetResolver {
  late String targettedField;
  late String targettedValue;

  TargetResolver({
    required this.targettedField,
    required this.targettedValue,
  });

  TargetResolver.fromJson(dynamic input) {
    targettedField = input['targettedField'];
    targettedValue = input['targettedValue'].toString();
  }
}

enum FieldType {
  text,
  textarea,
  phone,
  date,
  shortDate,
  number,
  zipCode,
  select,
  complex,
  amount,
}